package com.fjhx.oa.service.salary.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.oa.entity.salary.dto.SalaryStructureDto;
import com.fjhx.oa.entity.salary.dto.SalaryStructureSelectDto;
import com.fjhx.oa.entity.salary.po.SalaryStructure;
import com.fjhx.oa.entity.salary.vo.SalaryStructureVo;
import com.fjhx.oa.entity.user.vo.UserSalaryDetailVo;
import com.fjhx.oa.mapper.salary.SalaryStructureMapper;
import com.fjhx.oa.service.salary.SalaryStructureService;
import com.fjhx.oa.service.user.UserSalaryDetailService;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.wrapper.IWrapper;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;


/**
 * <p>
 * 薪资结构配置 服务实现类
 * </p>
 *
 * @author
 * @since 2024-06-11
 */
@Service
public class SalaryStructureServiceImpl extends ServiceImpl<SalaryStructureMapper, SalaryStructure> implements SalaryStructureService {

    @Resource
    private UserSalaryDetailService userSalaryDetailService;

    @Override
    public Page<SalaryStructureVo> getPage(SalaryStructureSelectDto dto) {
        IWrapper<SalaryStructure> wrapper = getWrapper();
        wrapper.orderByDesc("ss", SalaryStructure::getId);
        Page<SalaryStructureVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public SalaryStructureVo detail(Long id) {
        SalaryStructure SalaryStructure = this.getById(id);
        SalaryStructureVo result = BeanUtil.toBean(SalaryStructure, SalaryStructureVo.class);
        return result;
    }

    @Override
    public void add(SalaryStructureDto salaryStructureDto) {
        this.save(salaryStructureDto);
    }

    @Override
    public void edit(SalaryStructureDto salaryStructureDto) {
        this.updateById(salaryStructureDto);
    }

    @Override
    public void delete(Long id) {
        List<UserSalaryDetailVo> list = userSalaryDetailService.getList(IWrapper.getWrapper().eq("usd.salary_structure_id", id));
        if (ObjectUtil.isNotEmpty(list)) {
            throw new ServiceException("该薪资结构已被使用，禁止删除");
        }
        this.removeById(id);
    }

}
