package com.fjhx.sale.flow;

import com.alibaba.fastjson.JSONObject;
import com.fjhx.common.enums.FlowStatusEnum1;
import com.fjhx.flow.core.FlowDelegate;
import com.fjhx.flow.enums.FlowStatusEnum;
import com.fjhx.flow.enums.NodeTypeEnum;
import com.fjhx.item.entity.product.po.ProductInfo;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.sale.entity.contract.dto.ContractProductDto;
import com.fjhx.sale.entity.contract.po.Contract;
import com.fjhx.sale.entity.contract.po.ContractProduct;
import com.fjhx.sale.entity.contract.po.ContractProductBom;
import com.fjhx.sale.service.contract.ContractProductBomService;
import com.fjhx.sale.service.contract.ContractProductService;
import com.fjhx.sale.service.contract.ContractService;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;

/**
 * 订单评审流程
 */
@Component
public class ContractReviewFlow extends FlowDelegate {

    private final ContractService contractService;
    private final ContractProductService contractProductService;
    private final ContractProductBomService contractProductBomService;
    private final ProductInfoService productInfoService;

    public ContractReviewFlow(ContractService contractService, ContractProductService contractProductService, ContractProductBomService contractProductBomService, ProductInfoService productInfoService) {
        this.contractService = contractService;
        this.contractProductService = contractProductService;
        this.contractProductBomService = contractProductBomService;
        this.productInfoService = productInfoService;
    }

    @Override
    public String getFlowKey() {
        return "contract_review_flow";
    }

    @Override
    public Long start(Long flowId, JSONObject submitData) {
        ContractProductDto dto = submitData.toJavaObject(ContractProductDto.class);

        //赋值流程信息
        contractProductService.update(q -> q
                .eq(ContractProduct::getId, dto.getId())
                .set(ContractProduct::getReviewStatus, FlowStatusEnum1.UNDER_REVIEW.getKey())
                .set(ContractProduct::getReviewFlowId, flowId)
        );

        Contract contract = contractService.getById(dto.getContractId());
        //赋值产品信息
        ProductInfo productInfo = productInfoService.getById(dto.getProductId());

        submitData.put("productCode", productInfo.getCustomCode());
        submitData.put("contractCode", contract.getCode());
        submitData.put("prodCompanyId", contract.getCompanyId());
        submitData.put("salesmanId", contract.getSalesmanId());


        return dto.getId();
    }

    @Override
    public void defaultMethod(Long flowId, Long businessId, NodeTypeEnum handleNodeType, FlowStatusEnum flowStatus, JSONObject submitData) {
        //修改数据
        ContractProductDto dto = submitData.toJavaObject(ContractProductDto.class);
        contractProductService.updateById(dto);
        List<ContractProductBom> contractProductBomList = dto.getContractProductBomList();
        contractProductBomService.updateBatchById(contractProductBomList);

        //执行原本的方法
        super.defaultMethod(flowId, businessId, handleNodeType, flowStatus, submitData);
    }

    @Override
    public void reject(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        contractProductService.update(q -> q
                .eq(ContractProduct::getId, businessId)
                .set(ContractProduct::getReviewStatus, FlowStatusEnum1.REJECT.getKey())
        );
    }

    @Override
    public void cancellation(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        contractProductService.update(q -> q
                .eq(ContractProduct::getId, businessId)
                .set(ContractProduct::getReviewStatus, FlowStatusEnum1.CANCELLATION.getKey())
        );
    }

    @Override
    public void end(Long flowId, Long businessId, JSONObject submitData) {
        contractProductService.update(q -> q
                .eq(ContractProduct::getId, businessId)
                .set(ContractProduct::getReviewStatus, FlowStatusEnum1.PASS.getKey())
        );

        //所有产品都评审 将合同改为已评审
        ContractProduct contractProduct = contractProductService.getById(businessId);
        long count = contractProductService.count(q -> q
                .eq(ContractProduct::getContractId, contractProduct.getContractId())
                .ne(ContractProduct::getReviewStatus, FlowStatusEnum1.PASS.getKey())
        );
        if (count == 0) {
            contractService.update(q -> q
                    .eq(Contract::getId, contractProduct.getContractId())
                    .set(Contract::getReviewStatus, FlowStatusEnum1.PASS.getKey())
                    .set(Contract::getReviewEndTime, new Date())
            );
        }
    }
}
