package com.fjhx.wms.service.manual.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.tenant.utils.DeptUstil;
import com.fjhx.wms.entity.manual.dto.ManualStockDetailsSelectDto;
import com.fjhx.wms.entity.manual.po.ManualStockDetails;
import com.fjhx.wms.entity.manual.vo.ManualStockDetailsVo;
import com.fjhx.wms.mapper.manual.ManualStockDetailsMapper;
import com.fjhx.wms.service.manual.ManualStockDetailsService;
import com.fjhx.wms.service.warehouse.WarehouseService;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.common.utils.wrapper.SqlField;
import com.ruoyi.system.utils.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;


/**
 * <p>
 * 手动出入库记录明细 服务实现类
 * </p>
 *
 * @author
 * @since 2024-04-24
 */
@Service
public class ManualStockDetailsServiceImpl extends ServiceImpl<ManualStockDetailsMapper, ManualStockDetails> implements ManualStockDetailsService {

    @Autowired
    private WarehouseService warehouseService;

    @Override
    public Page<ManualStockDetailsVo> getPage(ManualStockDetailsSelectDto dto) {
        IWrapper<ManualStockDetails> wrapper = getWrapper();

        wrapper.eq("ms.warehouse_id", dto.getWarehouseId());
        wrapper.eq("ms.type", dto.getType());
        wrapper.eq("ms.op_type", dto.getOpType());
        wrapper.in(ObjectUtil.isNotEmpty(dto.getProductIds()), "msd.product_id", dto.getProductIds());
        wrapper.eq("pi.definition", dto.getDefinition());
        wrapper.eq("pi.product_classify_id", dto.getProductClassifyId());
        wrapper.eq("ms.status", dto.getStatus());


        wrapper.keyword(dto.getKeyword(), new SqlField("pi.name"), new SqlField("pi.custom_code"));

        //权限过滤：库存
        wrapper.in("ms.company_id", SecurityUtils.getCompanyIds());
        wrapper.eq("ms.company_id", dto.getCompanyId());

        wrapper.orderByDesc("ms", ManualStockDetails::getCreateTime);
        wrapper.orderByDesc("msd", ManualStockDetails::getId);


        Page<ManualStockDetailsVo> page = baseMapper.getPage(dto.getPage(), wrapper);
        steInfo(page.getRecords());
        return page;
    }

    private void steInfo(List<ManualStockDetailsVo> list) {
        if (ObjectUtil.isEmpty(list)) {
            return;
        }
        UserUtil.assignmentNickName(list, ManualStockDetailsVo::getCreateUser, ManualStockDetailsVo::setOpUserName);
        DeptUstil.assignmentNickName(list, ManualStockDetailsVo::getCompanyId, ManualStockDetailsVo::setCompanyName);
        warehouseService.attributeAssign(list, ManualStockDetailsVo::getWarehouseId, (item, warehouse) -> {
            item.setWarehouseName(warehouse.getName());
        });
    }
}
